#
# Sensus Consulting Ltd (C) 1997
# Matt Newman <matt@sensus.org>
#
# Base Class for lightweight canvas compound objects
#
#INDEX\
proc tk++::Canvas::Object {} {}

package require tk++

class tk++::Canvas::Object {

	constructor {args} {eval configure $args}

	public {
		method delete {tag} {}
		method coords {tag args} {error "abstract method"}
		method create {type x y args} {}
		method itemcget {tag option} {}
		method itemconfigure {tag args} {}
		method move {tag dx dy} {}

		variable canvas ""
	}

	# Virtual
	protected {
		method newTag {} {}
		method refresh {tag x y} {}

		variable items
		variable tags
		variable ox
		variable oy
		variable uid 0
	}
}
body tk++::Canvas::Object::delete {tag} {
	unset ox($tag)
	unset oy($tag)
	unset items($tag)
	foreach rtag [array names tags ${tag},*] {
		$canvas delete $tags($rtag)
		unset tags($rtag)
	}
}
body tk++::Canvas::Object::move {tag dx dy} {
	$canvas move $tag $dx $dy
	incr ox($tag) $dx
	incr oy($tag) $dy
}
body tk++::Canvas::Object::create {type x y args} {
	set tag [newTag]
	set ox($tag) $x
	set oy($tag) $y
	if {[llength $args]>0} {
		eval itemconfigure $tag $args
	}
	return $tag
}
body tk++::Canvas::Object::itemcget {tag option} {
	return [pget $items($tag) $option]
}
body tk++::Canvas::Object::itemconfigure {tag args} {
	set argc [llength $args]
	if {$argc == 0} {
		return $items($tag)
	} elseif {$argc == 1} {
		return [pget $items($tag) [lindex $args 0]]
	} else {
		set items($tag) [eval [list pset $items($tag)] $args]
		[info class]::refresh $tag $ox($tag) $oy($tag)
		return ""
	}
}
body tk++::Canvas::Object::newTag {} {
	set tag [info class][incr uid]
	set items($tag) {}
	return $tag
}
