#
# Sensus Consulting Ltd, Copyright 1997-1998
#
# Matt Newman <matt@sensus.org>
#
#
# file://		- returns list of volumes
#
namespace eval url {}
namespace eval url::ldap {}

proc url::ldap {option args} {
	variable ldap::methods
	variable ldap::ldap
	if [info exists methods($option)] {
		return [uplevel 1 url::ldap::$option $args]
	}
	return [uplevel 1 ${ldap::ldap} $option $args]
}
namespace eval url::ldap {
	variable methods
	variable ldap [ldap]

	set methods(configure)		1
	set methods(children)		1
	set methods(tail)			1
	set methods(join)			1
	set methods(delete)			1
	set methods(copy)			1

	proc icon {dn {default ""}} {
		set class [lindex [::split $dn =] 0]
		return [img ldap-$class folder]
	}
}
proc url::ldap::init {host} {
	variable ldap
	set ldap [ldap $host]
	$ldap bind
}
;proc url::ldap::configure {dn} {
	variable icons
	variable ldap

	if {$dn == ""} {
		return [list -name "LDAP Base DN" -image [img ldap-c] -container 1]
	}
	set ret {}
	lappend ret -name [lindex [$ldap split $dn true] 0]
	lappend ret -image [icon [lindex [$ldap split $dn false] 0]]
	lappend ret -container 1
	set attrs [lindex [$ldap search $dn] 0]
	foreach {fld val} $attrs {
		lappend ret $fld $val
	}
	return $ret
}
;proc url::ldap::children {dn {glob *}} {
	variable ldap

	if {$dn == ""} {
		set ret [lindex [$ldap search $dn] 0]
		if {[catch {pget $ret namingcontexts} nc]} {
			return ""
		}
		return $nc
	}
	set ret {}
	foreach cdn [$ldap search -scope one -names 1 $dn] {
		set cdn [tail $cdn]
		if {[string match $glob $cdn]} {
			lappend ret $cdn
		}
	}
	return $ret
}
;proc url::ldap::join {dn rdn} {
	variable ldap
	# Little-Endian
	if {$dn == ""} {return $rdn}
	return [$ldap join [list $rdn $dn]]
}
;proc url::ldap::tail {dn} {
	variable ldap
	# Little-Endian
	return [lindex [$ldap split $dn] 0]
}
;proc url::ldap::delete {dn} {
	variable ldap
	# Little-Endian
	foreach cdn [$ldap search -scope one -names 1 $dn] {
		delete $cdn
	}
	$ldap delete $dn
}
;proc url::ldap::copy {src dest} {
	variable ldap
	set data [lindex [$ldap search $src] 0]
	# Skip past dn
	set data [lrange $data 2 end]
	catch {
		set data [pdel $data aci]
	}
	eval [list $ldap add $dest] $data

	# Now copy children
	set n 1
	foreach cdn [$ldap search -scope one -names 1 $src] {
		set rdn [tail $cdn]
		incr n [copy [join $src $rdn] [join $dest $rdn]]
	}
	return $n
}
